'use strict';

const KubeManager = require('./');

const config = {
    "url": process.env['KUBE_HOST'],
    "auth": {
        "bearer": Buffer.from(process.env['KUBE_ACCOUNT_TOKEN'] || '', 'base64')
    },
    "ca": Buffer.from(process.env['KUBE_CA_CERT'] || '', 'base64'),
    "type": "sat"
};

/**
 * Init K8S manager
 * @type {*}
 */
const manager = new KubeManager(config);
const namespace = 'default';

//Get all service from default namespace
const ctrl = manager.getServiceController(namespace);
ctrl.get()
    .map(svc => {
        console.log(`All services from ${namespace}`);
        console.log(svc.toJson())
    });

// Create new Project
// Read more about project in README.md
const options           = {
    service: {
        metadata: {
            name: 'mongo'
        },
        spec: {
            ports: [
                {
                    port: 27017,
                    targetPort: 27017,
                    name: 'http'
                }
            ]
        }
    },
    deployment: {
        "metadata": {
            "name": "mongo"
        },
        "spec": {
            "template": {
                "spec": {
                    "containers": [{
                        "image": "mongo",
                        ports: [
                            {
                                containerPort: 27017
                            }
                        ]
                    }
                    ]
                }
            }
        }
    }
};
const projectController = KubeManager.projectController;
projectController.create({
    kubeManager: manager,
    ns: namespace,
    data: options
})
    .tap((project) => {
        console.log("Project created");
        console.log(project.toJson());
    })
    .then((project) => {
        console.log("Deleting project");
        projectController.delete({
            kubeManager: manager,
            project
        })
            .then(() => console.log("Project deleted"));
    });