"use strict";

const logging       = require('./logging');
const config  = require('../config');
const monitor = require('@codefresh-io/cf-monitor');
monitor.init();
const Promise       = require('bluebird'); // jshint ignore:line
const processEvents = require('./process-events');
const express       = require('./express');
const eventbus      = require('./eventbus');
const logger        = require('cf-logs').Logger('codefresh:infra:index');

class Microservice {

    constructor() {}

    init() {
        return logging.init(config)
            .then(() => {
                return processEvents.init(config)
                    .then(() => {
                        processEvents.on('SIGTERM', this.stop.bind(this));
                    });
            })
            .then(() => {
                return Promise.all([
                    eventbus.init(config)
                ]);
            })
            .then(() => {
                return express.init(config);
            })
            .then(() => {
                console.log(`Initialization completed`);
            })
            .catch((err) => {
                console.error(`Initialization error: ${err.stack}`);
                process.exit(1);
            })
            .done();
    }

    // TODO need to split shutdown process in to 2 phases.
    // - first phase need to make sure to not accept any new requests/events
    // - then a decent amount of time will be given to clear all on-going contexts
    // - second phase will close all dependencies connections like mongo, postgres etc
    stop() {
        logger.info('Starting shutdown...');
        return Promise.all([
            express.stop(),
            eventbus.stop()
        ])
            .then(() => {
                logger.info('Shutdown completed, exiting');
            });
    }

}

module.exports = new Microservice();
