'use strict';

const _ = require('lodash');
const CFError = require('cf-errors');

class BaseBuilder {
    constructor(descriptor, template, opt) {
        this._descriptor = descriptor;
        this._template   = template;
        this._opt        = opt;
        this._startBuild();
    }

    getDescriptor() {
        return this._descriptor;
    }

    updateTemplate(path, value) {
        const old    = _.get(this, `_template[${path}]`);
        const merged = old ? _.merge(old, value) : value;
        _.set(this._template, path, merged);
    }

    build() {
        return this._template;
    }

    shouldMarkWithLabels() {
        return _.get(this, '_opt.markWithLabels', true);
    }

    _shouldMarkAnnotations(){
        return _.get(this, '_opt.markAnnotations', true);
    }

    /**
     * Abstract method that should be implemented in sub-classes
     * @private
     */
    _startBuild() {
        throw new CFError('Not implemented');
    }

}

module.exports = BaseBuilder;